
Imports System.Drawing

Namespace System.Windows.Forms

   Public Class Sizer
      Private _SizeablePanel As SizeablePanel                                                     'mein Owner
      Private _MouseInSplitterAmount As Integer   'Abstand der Maus zum linken oder oberen Rand des Splitters
      Private _PanelMovesHorizontal As Boolean = False
      Private _MoveBorders(1) As Integer                                       'Grenzwerte frde Verschiebung
      Friend SizeRedraw As Boolean = True      'quasi Backing-Field der gleichnamigen _SizeablePanel-Property
      Private Shared ReadOnly _Splitter As Control = CreateSplitter()           'Wow - ein Singleton-Control!

#Region "Initialisation"

      Public Sub New(ByVal Pnl As SizeablePanel)
         Me._SizeablePanel = Pnl
      End Sub

      Private Shared Function CreateSplitter() As Control
         CreateSplitter = New Control
         With CreateSplitter
            .BackColor = Color.Yellow
            .Name = "_Splitter"
            .Text = .Name
            .Anchor = AnchorStyles.Top Or AnchorStyles.Left
            .Dock = DockStyle.None
            .Visible = True
         End With
      End Function

#End Region 'Initialisation

#Region "Publix"

      ''' <summary>aufgerufen von OnMouseDown</summary>
      Public Sub InitDrag(ByVal ptMouse As Point)
         ' _SizeablePanel ausmessen, _Splitter ausrichten, ggfs anzeigen, Move-Grenzwerte ermitteln
         Dim SplitBounds As Rectangle
         With _SizeablePanel
            Select Case .Dock
               Case DockStyle.Bottom
                  _PanelMovesHorizontal = False
                  SplitBounds = New Rectangle(0, 0, .Width, .SplitterWidth)

               Case DockStyle.Top
                  _PanelMovesHorizontal = False
                  SplitBounds = New Rectangle(0, .Height - .SplitterWidth, .Width, .SplitterWidth)

               Case DockStyle.Left
                  _PanelMovesHorizontal = True
                  SplitBounds = New Rectangle(.Width - .SplitterWidth, 0, .SplitterWidth, .Height)

               Case DockStyle.Right
                  _PanelMovesHorizontal = True
                  SplitBounds = New Rectangle(0, 0, .SplitterWidth, .Height)

               Case Else
                  Throw Me.Exception( _
                     _SizeablePanel.Name, ": kann nur sizen, wenn ich horizontal oder vertikal gedockt bin!")

            End Select
         End With

         If _PanelMovesHorizontal Then
            _MouseInSplitterAmount = ptMouse.X - SplitBounds.Left
         Else
            _MouseInSplitterAmount = ptMouse.Y - SplitBounds.Top
         End If

         'Begrenzungen ermitteln - dazu mu die Layout-Logik antizipiert werden.

         Static ParentAllControls As New Enmrt(Of Control)( _
            _SizeablePanel.Parent.Controls, Function(Cl) Cl.Controls) 'Ein Baum-Enumerierer, der alle 
         '                                               Controls des _SizeablePanel-Parents enthlt
         Dim ctlEnd As Control = Nothing 'das in der Controls-Auflistung dem _SizeablePanel folgende
         With _SizeablePanel.Parent.Controls
            Dim Indx As Integer = .IndexOf(_SizeablePanel)
            If Indx < .Count - 1 Then ctlEnd = .Item(Indx + 1)
         End With
         Const MinAmount As Integer = 4

         Dim MoveChecker As New Extremum(Of Integer)(GetMaxGrow)  'sammelt in seinem Minimum zunchst die
         '!                        erlaubte Vergrerung, dann die erlaubte Verkleinerung des _SizeablePanels
         Dim AllowedGrowing As Integer                               'erlaubte Vergrerung wird hier geparkt

         'Die Reihenfolge der Controls in der ControlCollection bestimmt, was im Vordergrund ist. Bei
         '! gedockten Controls verschieben die hinteren die Vorderen. Bei mehreren Links-gedockten Controls
         '! rckt also das vorderste ganz nach rechts.
         'Das Splitter-prinzip der SizeablePanels funzt einfach, indem Panel.Width verstellt wird.
         'Die maximale Vergrerung wird durch das Control rechts vom Panel bestimmt, welches am wenigsten
         '! Spielraum zu schrumpfen hat. 
         'Unter folgenden Bedingungen also reagiert ein Control flexibel auf Bettigen des Splitters:
         '     {
         '- es ist im SizeablePanel enthalten
         'oder
         '- es liegt in der ControlCollection *vorm* SizeablePanel
         'oder
         '- es ist in einem vorm SizeablePanel liegenden, *flexiblen* Control enthalten       }
         'und {
         '- ist mit anderer Orientierung gedockt (Top/Bottom vs. Left/Right)
         'oder
         '- hat Dockstyle.Fill      }
         'Der Algorithmus enumeriert alle Controls des Parents des SizeablePanels, von 0 bis zum
         '! SizeablePanel selbst. Die ihm folgenden brauchen nicht bercksichtigt zu werden. Denn: Ein links-
         '!gedocktes Control wchst nur nach rechts, nach links schrumpft es (maximal, bisses weg ist)
         'Die eine Splitter-Grenze findet sich also in demjenigen vorm SizeablePanel liegenden Control mit
         '! dem geringsten - Schrumpf-Spielraum, und die andere Splitter-Grenze in dem innerhalb von ihm
         '! liegenden schrumpf-schwchsten Control.
         For Each Ctl As Control In ParentAllControls
            If Ctl Is _SizeablePanel Then
               'bis hierher wurde alle vorm SizeablePanel liegenden Controls enumeriert
               AllowedGrowing = MoveChecker.MinVal                   'erlaubte Vergrerung wird hier geparkt
               'AllowedMove neu initialisieren - ab jetzt wird das SizeablePanel-Schrumpf-Maximum ermittelt
               If _PanelMovesHorizontal Then
                  With _SizeablePanel
                     MoveChecker.Reset(.Width - (.SplitterWidth + .MinimumSize.Width + MinAmount))     'mittm
                     '!                              "Schrumpf-Vermgen" des SizeablePanels re-initialisieren
                  End With
               Else
                  With _SizeablePanel
                     MoveChecker.Reset(.Height - (.SplitterWidth + .MinimumSize.Height + MinAmount))
                  End With
               End If
               Continue For
            End If

            If Ctl Is ctlEnd Then Exit For 'fertig - weitere Controls nicht bercksichtigen

            If Not Ctl.Visible OrElse Ctl.Dock = DockStyle.None Then
               ParentAllControls.SkipChildren = True         'SkipChildren berspringt den ganzen Tree-Zweig
               Continue For
            End If

            If _PanelMovesHorizontal Then
               Select Case Ctl.Dock
                  Case DockStyle.Left, DockStyle.Right
                     'ausschlieen: gleichsinnig gedockte Controls schieben sich nicht zusammen
                     ParentAllControls.SkipChildren = True    'darin enthaltene auch berspringen
                     Continue For
               End Select
               MoveChecker.CheckNew(Ctl.Width - (MinAmount + Ctl.MinimumSize.Width)) 'das "Schrumpf-Vermgen"
               '!                                                                                   auswerten
            Else
               Select Case Ctl.Dock
                  Case DockStyle.Bottom, DockStyle.Top
                     ParentAllControls.SkipChildren = True
                     Continue For
               End Select
               MoveChecker.CheckNew(Ctl.Height - (MinAmount + Ctl.MinimumSize.Height))
            End If

            If TypeOf Ctl Is TableLayoutPanel Then
               ' TableLayoutPanels LayoutLogik kannich nicht antizipieren - das sind ja u.U. anteil-werte,
               '! um die die auf einem TableLayoutPanel aufsitzenden Controls grengendert werden.
               With DirectCast(Ctl, TableLayoutPanel)
                  If .MinimumSize = Global.System.Drawing.Size.Empty Then
                     Dbg("Its a warning: \nI've detected a TableLayoutPanel, ", _
                        "wich can be shrinked upto nothing. Is it this, what you wanted?")
                  End If
               End With
               ParentAllControls.SkipChildren = True
               Continue For
            End If

         Next
         If Not SizeRedraw Then ShowSplitter(SplitBounds) 'erst jetzt das Splitter-Control adden, (muss ja
         '!                                                                          nicht enumeriert werden)

         ' AllowedShrinking, AllowedGrowing in ein korrekt sortiertes Grenzwertepaar bertragen
         ptMouse.Offset(_SizeablePanel.Location) 'Die Grenzwerte beziehen sich auf SizeablePanels Parent
         Select Case _SizeablePanel.Dock
            Case DockStyle.Left
               _MoveBorders(0) = ptMouse.X - MoveChecker.MinVal  ' (MoveChecker.MinVal = "AllowedShrinking")
               _MoveBorders(1) = ptMouse.X + AllowedGrowing

            Case DockStyle.Top
               _MoveBorders(0) = ptMouse.Y - MoveChecker.MinVal
               _MoveBorders(1) = ptMouse.Y + AllowedGrowing

            Case DockStyle.Right
               _MoveBorders(0) = ptMouse.X - AllowedGrowing
               _MoveBorders(1) = ptMouse.X + MoveChecker.MinVal

            Case DockStyle.Bottom
               _MoveBorders(0) = ptMouse.Y - AllowedGrowing
               _MoveBorders(1) = ptMouse.Y + MoveChecker.MinVal

         End Select
      End Sub

      ''' <summary>aufgerufen von OnMouseMove</summary>
      Public Sub MoveSplitter(ByVal Pt As Point)
         With _SizeablePanel
            Pt.Offset(.Location)
            If SizeRedraw Then                                 'direkt sizen - kein Splitter-Control anzeigen
               Static ptOld As New Point(Integer.MinValue, Integer.MinValue)
               If ptOld = Pt Then
                  'bei Dock.Bottom/Right wird _SizeablePanel der Maus unterm Arsch verschoben. Das fhrt zur
                  '! Verdopplung des MouseMoves, und filternwa hier raus.
                  Return
               Else
                  ptOld = Pt
               End If
               Select Case .Dock
                  Case DockStyle.Bottom
                     .Height = .Bottom - (ClipInRange(Pt.Y) - _MouseInSplitterAmount)

                  Case DockStyle.Right
                     .Width = .Right + _MouseInSplitterAmount - ClipInRange(Pt.X)

                  Case DockStyle.Top
                     .Height = ClipInRange(Pt.Y) + .SplitterWidth - (_MouseInSplitterAmount + .Top)

                  Case DockStyle.Left
                     .Width = ClipInRange(Pt.X) + .SplitterWidth - (_MouseInSplitterAmount + .Left)

               End Select
            Else          'der Modus mittm Splitter-Control programmiert sich viel eleganter, findich
               If _PanelMovesHorizontal Then
                  _Splitter.Left = ClipInRange(Pt.X) - _MouseInSplitterAmount
               Else
                  _Splitter.Top = ClipInRange(Pt.Y) - _MouseInSplitterAmount
               End If
            End If
         End With
      End Sub

      ''' <summary>aufgerufen von OnMouseUp</summary>
      Public Sub EndSize()
         If SizeRedraw Then Return 'im SizeRedraw-Modus hatterja schon gesized
         With _SizeablePanel
            If _Splitter.Parent Is _SizeablePanel.Parent Then
               .Parent.Controls.Remove(_Splitter)
            Else
               Throw Me.Exception(" EndSize - _Splitter not found ")
            End If
            Select Case .Dock
               Case DockStyle.Bottom
                  .Height = .Bottom - _Splitter.Top

               Case DockStyle.Top
                  .Height = _Splitter.Bottom - .Top

               Case DockStyle.Left
                  .Width = _Splitter.Right - .Left

               Case DockStyle.Right
                  .Width = .Right - _Splitter.Left

            End Select
         End With
      End Sub

#End Region 'Publix

#Region "Helpers"

      Private Sub ShowSplitter(ByVal Rct As Rectangle)
         'der Splitter wird dem *Parent* des _SizeablePanel hinzugefgt
         Rct.Offset(_SizeablePanel.Location)
         _Splitter.Bounds = Rct
         _SizeablePanel.Parent.Controls.Add(_Splitter)
         _Splitter.BringToFront()
      End Sub

      ''' <summary>
      ''' gibt den maximalen Wert, um den sich  _SizeablePanel vergrern darf, ohne seines Parents Mae 
      ''' zu transzendieren
      ''' </summary>
      Private Function GetMaxGrow() As Integer
         With _SizeablePanel
            Select Case .Dock
               Case DockStyle.Left
                  Return .Parent.ClientSize.Width - .Right

               Case DockStyle.Right
                  Return .Left

               Case DockStyle.Top
                  Return .Parent.ClientSize.Height - .Bottom

               Case DockStyle.Bottom
                  Return .Top

            End Select
         End With
      End Function

      Private Function ClipInRange(ByVal Val As Integer) As Integer
         If Val < _MoveBorders(0) Then Return _MoveBorders(0)
         If Val > _MoveBorders(1) Then Return _MoveBorders(1)
         Return Val
      End Function

#End Region 'Helpers

   End Class


End Namespace