Namespace System
   ''' <summary>
   ''' QueryHelper-Derivate knnen Listen und Arrays durchsuchen und sortieren, wobei Such-Matches und 
   ''' Sortier-Comparisons in berschreibungen frei programmierbar sind.
   ''' </summary>
   ''' <typeparam name="TDerivate">Typ des Derivates selbst</typeparam>
   ''' <typeparam name="TItem">Typ der zu durchsuchende/sortierenden Item</typeparam>
   ''' <typeparam name="TProperty">Typ der Item-Property, nach der zu durchsuchen/sortieren ist</typeparam>
   ''' <remarks>
   ''' Bsp.: Public Class PointXQuery : Inherits QueryHelper(Of PointXQuery, Point, Integer)
   ''' </remarks>
   Public Class QueryHelper(Of _
         TDerivate As {QueryHelper(Of TDerivate, TItem, TProperty), New}, _
         TItem, _
         TProperty)
      Implements IComparer, IComparer(Of TItem)   'fr [Array/List(of TItem)].BinarySearch erf. Interface

      Private Shared Singleton As New TDerivate
      Private Shared KeyValue As TProperty

#Region "Overridables"

      ''' <summary>untersttzt Sortierung</summary>
      Protected Overridable Function CompareItems(ByVal Item0 As TItem, ByVal Item1 As TItem) As Integer
         Throw Me.Exception( _
            "Sortierung kann nicht erfolgen: Die CompareItems-Function wurde nicht berschrieben")
      End Function
      ''' <summary>untersttzt Suchen, Filtern, BinarySearch</summary>
      Protected Overridable Function CompareKey(ByVal Item As TItem, ByVal Key As TProperty) As Integer
         Throw Me.Exception( _
            "Suche/Filterung/binre Suche kann nicht erfolgen: Die CompareKey-Function wurde ", _
            "nicht berschrieben")
      End Function
      ''' <summary>untersttzt Suchen und Filtern</summary>
      Protected Overridable Function IsMatch(ByVal Item As TItem, ByVal Key As TProperty) As Boolean
         Return CompareKey(Item, KeyValue) = 0
      End Function

#End Region 'Overridables

#Region "Privates"

      ''' <summary>untersttzt Suchen und Filtern</summary>
      Private Function IsMatchPredicate(ByVal Item As TItem) As Boolean
         Return IsMatch(Item, KeyValue)
      End Function

      Private Function CompareForBinarySearch(ByVal x As TItem, ByVal y As TItem) As Integer _
            Implements System.Collections.Generic.IComparer(Of TItem).Compare
         Return CompareKey(x, KeyValue)
      End Function

      Private Function CompareForBinarySearch(ByVal x As Object, ByVal y As Object) As Integer _
            Implements System.Collections.IComparer.Compare
         Return CompareForBinarySearch(x, KeyValue)
      End Function

#End Region 'Privates



#Region "Publix"

      Public Shared Sub Sort(ByVal Arr As TItem())
         Array.Sort(Arr, AddressOf Singleton.CompareItems)
      End Sub
      Public Shared Function Find(ByVal Arr As TItem(), ByVal Key As TProperty) As TItem
         KeyValue = Key                                                       'KeyValue im Shared Feld merken
         Return Array.Find(Arr, AddressOf Singleton.IsMatchPredicate)
      End Function
      Public Shared Function FindAll(ByVal Arr As TItem(), ByVal Key As TProperty) As TItem()
         KeyValue = Key
         Return Array.FindAll(Arr, AddressOf Singleton.IsMatchPredicate)
      End Function


      Public Shared Sub Sort(ByVal List As List(Of TItem))
         List.Sort(AddressOf Singleton.CompareItems)
      End Sub
      Public Shared Function Find(ByVal List As List(Of TItem), ByVal Key As TProperty) As TItem
         KeyValue = Key
         Return List.Find(AddressOf Singleton.IsMatchPredicate)
      End Function
      Public Shared Function FindAll(ByVal List As List(Of TItem), ByVal Key As TProperty) As List(Of TItem)
         KeyValue = Key
         Return List.FindAll(AddressOf Singleton.IsMatchPredicate)
      End Function

#Region "Publix-BinarySearch"
      'little dirty Hack: Der Vergleich findet nicht zwischen 2 TItem-Objekten statt, wie BinarySearch intern
      '! erwartet, wennes den Singleton per IComparer-Schnittstelle anspricht, sondern zw. einem TItem-Objekt
      '! und dem vorher gemerkten KeyValue. Wrde BinarySearch die Vergleichspartner in anderer Reihenfolge
      '! bergeben, ttes crashen. Mge MS also niemals den Algorithmus ndern!
      'Wem das Verfahren "binre Suche" nicht bekannt ist, dem empfehle ich, in der Wiki nachzuschlagen.
      Public Shared Function BinarySearch(ByVal Arr As TItem(), ByVal Key As TProperty) As Integer
         KeyValue = Key
         'hier wird statt eines T-Objektes Nothing in die Suche geschickt...
         Return Array.BinarySearch(Arr, Nothing, Singleton)
      End Function
      Public Shared Function BinarySearch(ByVal List As List(Of TItem), ByVal Key As TProperty) As Integer
         KeyValue = Key
         Return List.BinarySearch(Nothing, Singleton)
      End Function
#End Region 'Publix-BinarySearch

#End Region 'Publix

   End Class


End Namespace